//*****************************************************************************
//
// File Name	: 'mmc.h'
// Title		: MultiMedia and SD Flash Card Interface
// Author		: Pascal Stang - Copyright (C) 2004
// Created		: 2004.09.22
// Revised		: 2004.09.22
// Version		: 0.1
// Target MCU	: Atmel AVR Series
// Editor Tabs	: 4
//
//	\ingroup driver_hw
// \defgroup mmc MultiMedia and SD Flash Card Interface (mmc.c)
// \code #include "mmc.h" \endcode
// \par Description
//		This library offers some simple functions which can be used
//		to read and write data on a MultiMedia or SecureDigital (SD) Flash
//		Card.  Although MM and SD Cards are designed to operate with their own
//		special bus wiring and protocols, both types of cards also provide a
//		simple SPI-like interface mode which is exceptionally useful when 
//		attempting to use the cards in embedded systems.
//
// \par Wiring
//		To work with this library, the card must be wired to the SPI port of
//		the Atmel microcontroller as described below.  Typical cards can
//		operate at up to 25MHz maximum SPI clock rate (thus faster than most
//		AVR's maximum SPI clock rate).
// <pre>
//	      _________________
//	     /  1 2 3 4 5 6 78 |	<- view of MMC/SD card looking at contacts
//	    / 9                |	Pins 8 and 9 are present only on SD cards
//	    |    MMC/SD Card   |
//	    |                  |
//	    X X X X X X X X X X 
//	    1 - CS   (chip select)          - wire to any available I/O pin(*)
//	    2 - DIN  (data in, card<-host)  - wire to SPI MOSI pin
//	    3 - VSS  (ground)               - wire to ground
//	    4 - VDD  (power, 3.3V only?)    - wire to power (MIGHT BE 3.3V ONLY!)
//	    5 - SCLK (data clock)           - wire to SPI SCK pin
//	    6 - VSS  (ground)               - wire to ground
//	    7 - DOUT (data out, card->host) - wire to SPI MISO pin
//
//	    (*) you must define this chip select I/O pin in mmcconf.h
// </pre>
// \note This code is currently below version 1.0, and therefore is considered
// to be lacking in some functionality or documentation, or may not be fully
// tested.  Nonetheless, you can expect most functions to work.
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************
//-------------------------------------------------------------------------

#define SD_RAW_SDHC 1  //Enable SDHC Support

// constants/macros/typdefs
// MMC commands (taken from sandisk MMC reference)
#define MMC_GO_IDLE_STATE			0		//initialize card to SPI-type access
#define CMD_GO_IDLE_STATE 			0x00
#define CMD_SEND_OP_COND 			0x01
#define MMC_SEND_OP_COND			1		//set card operational mode
#define MMC_SEND_CSD				9		//get card's CSD
#define CMD_SEND_CSD 				0x09
#define MMC_SEND_CID				10		//get card's CID
#define MMC_SEND_STATUS				13
#define MMC_SET_BLOCKLEN			16		//Set num of bytes to transfer per block
#define MMC_READ_SINGLE_BLOCK		17		//read a block
#define MMC_WRITE_BLOCK				24		//write a block
#define MMC_PROGRAM_CSD				27
#define MMC_SET_WRITE_PROT			28
#define MMC_CLR_WRITE_PROT			29
#define MMC_SEND_WRITE_PROT			30
#define MMC_TAG_SECTOR_START		32
#define MMC_TAG_SECTOR_END			33
#define MMC_UNTAG_SECTOR			34
#define MMC_TAG_ERASE_GROUP_START 	35		//Sets begin of erase group (mass erase)
#define MMC_TAG_ERARE_GROUP_END		36		//Sets end of erase group (mass erase)
#define MMC_UNTAG_ERASE_GROUP		37		//Untag (unset) erase group (mass erase)
#define MMC_ERASE					38		//Perform block/mass erase
#define MMC_CRC_ON_OFF				59		//Turns CRC check on/off
#define CMD_SEND_IF_COND 			0x08
#define CMD_SD_SEND_OP_COND 		0x29
#define CMD_SET_BLOCKLEN 			0x10
#define CMD_SEND_CID 				0x0a

/* CMD55: arg0[31:0]: stuff bits, response R1 */
#define CMD_APP 0x37
/* CMD58: arg0[31:0]: stuff bits, response R3 */
#define CMD_READ_OCR 0x3a
/* CMD59: arg0[31:1]: stuff bits, arg0[0:0]: crc option, response R1 */
#define CMD_CRC_ON_OFF 0x3b
//-------------------------------------------------------------------------
// R1 Response bit-defines
#define MMC_R1_BUSY					0x80	//R1: bit indicates card is busy
#define MMC_R1_PARAMETER			0x40
#define MMC_R1_ADDRESS				0x20
#define MMC_R1_ERASE_SEQ			0x10
#define MMC_R1_COM_CRC				0x08
#define MMC_R1_ILLEGAL_COM			0x04
#define MMC_R1_ERASE_RESET			0x02
#define MMC_R1_IDLE_STATE			0x01
//-------------------------------------------------------------------------
// Data Start tokens
#define MMC_STARTBLOCK_READ			0xFE	//when received from card, indicates 
												//that a block of data will follow
#define MMC_STARTBLOCK_WRITE		0xFE	//when sent to card, indicates that a
												//block of data will follow
#define MMC_STARTBLOCK_MWRITE		0xFC
//-------------------------------------------------------------------------
// Data Stop tokens
#define MMC_STOPTRAN_WRITE			0xFD
//-------------------------------------------------------------------------
// Data Error Token values
#define MMC_DE_MASK					0x1F
#define MMC_DE_ERROR				0x01
#define MMC_DE_CC_ERROR				0x02
#define MMC_DE_ECC_FAIL				0x04
#define MMC_DE_OUT_OF_RANGE			0x04
#define MMC_DE_CARD_LOCKED			0x04
//-------------------------------------------------------------------------
// Data Response Token values
#define MMC_DR_MASK					0x1F
#define MMC_DR_ACCEPT				0x05
#define MMC_DR_REJECT_CRC			0x0B
#define MMC_DR_REJECT_WRITE_ERROR	0x0D
//-------------------------------------------------------------------------
/* command responses */
/* R1: size 1 byte */
#define R1_IDLE_STATE 0
#define R1_ERASE_RESET 1
#define R1_ILL_COMMAND 2
#define R1_COM_CRC_ERR 3
#define R1_ERASE_SEQ_ERR 4
#define R1_ADDR_ERR 5
#define R1_PARAM_ERR 6
/* R1b: equals R1, additional busy bytes */
/* R2: size 2 bytes */
#define R2_CARD_LOCKED 0
#define R2_WP_ERASE_SKIP 1
#define R2_ERR 2
#define R2_CARD_ERR 3
#define R2_CARD_ECC_FAIL 4
#define R2_WP_VIOLATION 5
#define R2_INVAL_ERASE 6
#define R2_OUT_OF_RANGE 7
#define R2_CSD_OVERWRITE 7
#define R2_IDLE_STATE (R1_IDLE_STATE + 8)
#define R2_ERASE_RESET (R1_ERASE_RESET + 8)
#define R2_ILL_COMMAND (R1_ILL_COMMAND + 8)
#define R2_COM_CRC_ERR (R1_COM_CRC_ERR + 8)
#define R2_ERASE_SEQ_ERR (R1_ERASE_SEQ_ERR + 8)
#define R2_ADDR_ERR (R1_ADDR_ERR + 8)
#define R2_PARAM_ERR (R1_PARAM_ERR + 8)
/* R3: size 5 bytes */
#define R3_OCR_MASK 		(0xffffffffUL)
#define R3_IDLE_STATE 		(R1_IDLE_STATE + 32)
#define R3_ERASE_RESET 		(R1_ERASE_RESET + 32)
#define R3_ILL_COMMAND 		(R1_ILL_COMMAND + 32)
#define R3_COM_CRC_ERR 		(R1_COM_CRC_ERR + 32)
#define R3_ERASE_SEQ_ERR 	(R1_ERASE_SEQ_ERR + 32)
#define R3_ADDR_ERR 		(R1_ADDR_ERR + 32)
#define R3_PARAM_ERR 		(R1_PARAM_ERR + 32)
/* Data Response: size 1 byte */
#define DR_STATUS_MASK 		0x0e
#define DR_STATUS_ACCEPTED 	0x05
#define DR_STATUS_CRC_ERR 	0x0a
#define DR_STATUS_WRITE_ERR 0x0c

/* status bits for card types */
#define SD_RAW_SPEC_1 		0
#define SD_RAW_SPEC_2 		1
#define SD_RAW_SPEC_SDHC 	2

#if SD_RAW_SDHC
    typedef uint64_t offset_t;
#else
    typedef uint32_t offset_t;
#endif

struct sd_raw_info
{ 
    uint8_t manufacturer;
    uint8_t oem[3];
    uint8_t product[6];
    uint8_t revision;
    uint32_t serial;   //
    uint8_t manufacturing_year;  //A value of zero means year 2000.
    uint8_t manufacturing_month; //The month of manufacturing
    offset_t capacity; //MB
    uint8_t flag_copy;
    uint8_t flag_write_protect;
    uint8_t flag_write_protect_temp;
    uint8_t format;
};

extern struct sd_raw_info  cardinf;
extern struct MMC_VOLUME_INFO vinf;
extern uint8_t sd_raw_card_type;
//-------------------------------------------------------------------------
// functions
unsigned char Card_IS_In(void);
//-------------------------------------------------------------------------
//! Initialize AVR<->MMC hardware interface.Prepares hardware for MMC access.
void mmcInit(void);
//-------------------------------------------------------------------------
//! Initialize the card and prepare it for use. Returns zero if successful.
uint8_t mmcReset(void);
//-------------------------------------------------------------------------
//! Send card an MMC command. Returns R1 result code.
uint8_t mmcSendCommand(uint8_t cmd, uint32_t arg);
//-------------------------------------------------------------------------
//! Read 512-byte sector from card to buffer Returns zero if successful.
uint8_t mmcRead(uint32_t sector, uint8_t* buffer);
//-------------------------------------------------------------------------
//! Write 512-byte sector from buffer to card Returns zero if successful.
uint8_t mmcWrite(uint32_t sector, uint8_t* buffer);
//-------------------------------------------------------------------------
//! Internal command function.Issues a generic MMC command.
uint8_t mmcCommand(uint8_t cmd, uint32_t arg);
//-------------------------------------------------------------------------
void MMC_get_volume_info(void);
//-------------------------------------------------------------------------
uint8_t sd_raw_get_info(struct sd_raw_info* info);
//-------------------------------------------------------------------------
